'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _requireHacker = require('require-hacker');

var _requireHacker2 = _interopRequireDefault(_requireHacker);

var _ethicalUtilityBrowserifier = require('ethical-utility-browserifier');

var _ethicalUtilityResolveModuleNode = require('ethical-utility-resolve-module-node');

var _ethicalUtilityPath = require('ethical-utility-path');

var _resolveCss = require('./resolve-css.js');

var _resolveCss2 = _interopRequireDefault(_resolveCss);

var _resolveJs = require('./resolve-js.js');

var _resolveJs2 = _interopRequireDefault(_resolveJs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

let moduleID = 0;
const cache = {};

const resolveModule = (request, parent) => {
    if (request.endsWith('.css')) {
        return (0, _resolveCss2.default)(request, parent);
    }
    return (0, _resolveJs2.default)(request, parent);
};

const resolveCache = (cached, modules, exclude) => {
    const { id, key, alias, source, path } = cached;

    if (typeof require.cache[path + '.*'] === 'object') {
        global.__ethical_server_middleware_module_supplier__[path] = require.cache[path + '.*'].exports;
    }

    if (exclude[id]) {
        const stubbedSource = `
            module.exports = (
                global.__ethical_server_middleware_module_supplier__['${path}']
            )
        `;
        return { source: stubbedSource, path };
    }

    modules.push(cached);
    exclude[id] = 1;
    return { source, path };
};

const handler = (modules, exclude) => (request, { filename: parent }) => {

    if (cache[request]) {
        return resolveCache(cache[request], modules, exclude);
    }

    const resolvedModule = resolveModule(request, parent);
    const { key, alias, path, source: getSource } = resolvedModule;

    if (cache[key]) {
        return resolveCache(cache[key], modules, exclude);
    }

    const source = getSource(path);
    const id = moduleID++;
    cache[key] = { id, key, alias, source, path };

    return resolveCache(cache[key], modules, exclude);
};

const captureModules = exclude => {
    const modules = [];
    const globalHook = _requireHacker2.default.global_hook('*', handler(modules, exclude));
    return () => {
        const cachedModules = modules.map(cached => {
            const { id, key, alias, source, path } = cached;

            if (typeof require.cache[path + '.*'] === 'object') {
                global.__ethical_server_middleware_module_supplier__[path] = require.cache[path + '.*'].exports;
            }

            return { id, key, alias, source };
        });
        globalHook.unmount();
        return cachedModules;
    };
};

const decacheNode = () => {
    Object.keys(require.cache).forEach(key => {
        delete require.cache[key];
    });
    global.__ethical_server_middleware_module_supplier__ = {};
};

const moduleCapturerMiddleware = (() => {
    var _ref = _asyncToGenerator(function* (ctx, next, config) {

        if (typeof ctx.response.body !== 'undefined') return yield next();

        const { main, path = 'module' } = config;

        if (ctx.request.path !== '/' + path) return yield next();

        const { request: { query: { entry = (0, _ethicalUtilityPath.absolute)(main), exclude = '' } } } = ctx;

        (0, _ethicalUtilityBrowserifier.browserify)();
        const excluded = {};
        exclude.split(',').forEach(function (id) {
            return excluded[id] = 1;
        });
        const releaseModules = captureModules(excluded);
        try {
            (0, _ethicalUtilityResolveModuleNode.requireModule)(entry);
        } catch (e) {
            console.error(e);
        }
        const modules = releaseModules();

        (0, _ethicalUtilityBrowserifier.unbrowserify)();

        const [mainModule = {}] = modules;

        if (main === (0, _ethicalUtilityPath.relative)(entry)) {
            mainModule.alias = mainModule.key;
            mainModule.key = (0, _ethicalUtilityResolveModuleNode.getAppPrefix)();
        }

        ctx.response.body = JSON.stringify(modules);
        ctx.response.set('Content-Type', 'application/json');

        yield next();
    });

    return function moduleCapturerMiddleware(_x, _x2, _x3) {
        return _ref.apply(this, arguments);
    };
})();

const moduleCapturerMiddlewareInit = (config = {}) => {
    decacheNode();
    return (() => {
        var _ref2 = _asyncToGenerator(function* (ctx, next) {
            return yield moduleCapturerMiddleware(ctx, next, config);
        });

        return function (_x4, _x5) {
            return _ref2.apply(this, arguments);
        };
    })();
};

exports.default = moduleCapturerMiddlewareInit;
//# sourceMappingURL=index.js.map